/*
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/
/* Umlaute in Langform wandeln */

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "entumlauten.h"

static unsigned char tbl[512];
static int init_done;

#define MKUCHAR(x) ((unsigned char) x)

static void 
init_uwan (void)
{
	unsigned int i;

	for (i = 0; i < 256; i++)
	{
		tbl[i] = (unsigned char) i;
	}

	tbl[MKUCHAR ('') * 2] = MKUCHAR ('u');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('U');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('a');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('A');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('o');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('O');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('e');
	tbl[MKUCHAR ('') * 2] = MKUCHAR ('s');
	tbl[MKUCHAR ('') * 2 + 1] = MKUCHAR ('s');

	init_done = 1;
}

char *
entumlauten1 (const char *from)
{
	static char buf[258];		/* die meisten Strings sind kurz */
	char *res;
	size_t l;

	if (!init_done)
		init_uwan ();
	l = strlen (from);
	if (l * 2 >= sizeof (buf))
	{
		char *to;

		to = malloc (l * 2 + 1);
		if (!to)
			return NULL;
		entumlauten_fast (from, to, l);
		res = strdup (to);
		free (to);
	}
	else
	{
		entumlauten_fast (from, buf, l);
		res = strdup (buf);
	}
	return res;
}

void 
entumlauten (const char *from, char *to)
{
	const unsigned char *f = (const unsigned char *) from;
	int i;
	unsigned char c;

	if (!init_done)
		init_uwan ();

	while (*f)
	{
		i = (*f++);
	   /* Tabellenzugriffe sind teuer, also vermeiden */
		if (i < 128)
			*to++ = i;
		else
		{
			i *= 2;
			*to++ = tbl[i++];
			c = tbl[i++];
			if (c != '\0')
				*to++ = c;
		}
	}
	*to = 0;
}

void
entumlauten_fast (const char *from, char *to, size_t l)
{
	const unsigned char *f = (const unsigned char *) from;
	int i;
	unsigned char c;

	if (!init_done)
		init_uwan ();

	if (!l)
	{
		*to=0;
		return;
	}

#define WORK { \
		i=(*f++); \
		if (i<128) *to++=i; \
		else { i *=2 ; *to++ = tbl[i++]; c=tbl[i++]; if (c) *to++=c; } \
	}

   /*
    * Duffs device - loop unrolling.
    */
	switch (l % 16)
	{
		while (*f)
		{
	case 0:
			WORK
	case 15:
			WORK
	case 14:
			WORK
	case 13:
			WORK
	case 12:
			WORK
	case 11:
			WORK
	case 10:
			WORK
	case 9:
			WORK
	case 8:
			WORK
	case 7:
			WORK
	case 6:
			WORK
	case 5:
			WORK
	case 4:
			WORK
	case 3:
			WORK
	case 2:
			WORK
	case 1:
			WORK
		}
	}
	*to = 0;
}
