%token ID MONTH DAY MERIDIAN NUMBER UNIT MUNIT SUNIT ZONE DAYZONE AGO
%{
	/* 	Steven M. Bellovin (unc!smb)			*/
	/*	Dept. of Computer Science			*/
	/*	University of North Carolina at Chapel Hill	*/
	/*	@(#)getdate.y	2.13	9/16/86 */

#include <sys/types.h>
#include <ctype.h>
#include <time.h>

#ifndef NULL
#define	NULL	0
#endif

#define daysec (24L*60L*60L)

	static int timeflag, zoneflag, dateflag, dayflag, relflag;
	static time_t relsec, relmonth;
	static int hh, mm, ss, merid, daylite;
	static int dayord, dayreq;
	static int month, day, year;
	static int ourzone;

#define AM 1
#define PM 2
#define DAYLIGHT 1
#define STANDARD 2
#define MAYBE    3
%}

%%
timedate: 		/* empty */
	| timedate item;

item:	tspec =
		{timeflag++;}
	| zone =
		{zoneflag++;}
	| dtspec =
		{dateflag++;}
	| dyspec =
		{dayflag++;}
	| rspec =
		{relflag++;}
	| nspec;

nspec:	NUMBER =
		{if (timeflag && dateflag && !relflag) year = $1;
		else {timeflag++;hh = $1/100;mm = $1%100;ss = 0;merid = 24;}};

tspec:	NUMBER MERIDIAN =
		{hh = $1; mm = 0; ss = 0; merid = $2;}
	| NUMBER ':' NUMBER =
		{hh = $1; mm = $3; merid = 24;}
	| NUMBER ':' NUMBER MERIDIAN =
		{hh = $1; mm = $3; merid = $4;}
	| NUMBER ':' NUMBER NUMBER =
		{hh = $1; mm = $3; merid = 24;
		daylite = STANDARD; ourzone = -($4%100 + 60*$4/100);}
	| NUMBER ':' NUMBER ':' NUMBER =
		{hh = $1; mm = $3; ss = $5; merid = 24;}
	| NUMBER ':' NUMBER ':' NUMBER MERIDIAN =
		{hh = $1; mm = $3; ss = $5; merid = $6;}
	| NUMBER ':' NUMBER ':' NUMBER NUMBER =
		{hh = $1; mm = $3; ss = $5; merid = 24;
		daylite = STANDARD; ourzone = -($6%100 + 60*$6/100);};

zone:	ZONE =
		{ourzone = $1; daylite = STANDARD;}
	| DAYZONE =
		{ourzone = $1; daylite = DAYLIGHT;};

dyspec:	DAY =
		{dayord = 1; dayreq = $1;}
	| DAY ',' =
		{dayord = 1; dayreq = $1;}
	| NUMBER DAY =
		{dayord = $1; dayreq = $2;};

dtspec:	NUMBER '/' NUMBER =
		{month = $1; day = $3;}
	| NUMBER '/' NUMBER '/' NUMBER =
		{month = $1; day = $3; year = $5;}
	| MONTH NUMBER =
		{month = $1; day = $2;}
	| MONTH NUMBER ',' NUMBER =
		{month = $1; day = $2; year = $4;}
	| NUMBER MONTH =
		{month = $2; day = $1;}
	| NUMBER MONTH NUMBER =
		{month = $2; day = $1; year = $3;};


rspec:	NUMBER UNIT =
		{relsec +=  60L * $1 * $2;}
	| NUMBER MUNIT =
		{relmonth += $1 * $2;}
	| NUMBER SUNIT =
		{relsec += $1;}
	| UNIT =
		{relsec +=  60L * $1;}
	| MUNIT =
		{relmonth += $1;}
	| SUNIT =
		{relsec++;}
	| rspec AGO =
		{relsec = -relsec; relmonth = -relmonth;};
%%

time_t dayconv (int para_ord, int para_day, time_t para_now);
time_t timeconv (int para_hh, int para_mm, int para_ss, int para_mer);
time_t dateconv (int para_mm, int para_dd, int para_yy, int para_h,
	 int para_m, int para_s, int para_mer, int para_zone, int para_dayflag);
time_t monthadd (time_t para_sdate, time_t para_relmonth);
time_t daylcorr (time_t future, time_t now);
int lookup (const char *id);
int yyerror (const char *s);

static int mdays[12] =
{31, 0, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

#define epoch 1970

time_t 
dateconv (int para_mm, int para_dd, int para_yy, int para_h,
		  int para_m, int para_s,
		  int para_mer, int para_zone, int para_dayflag)
{
	time_t tod,
	 jdate;
	register int i;

	if (para_yy < 0)
		para_yy = -para_yy;
	if (para_yy < 100)
		para_yy += 1900;
	mdays[1] = 28 + (para_yy % 4 == 0 && (para_yy % 100 != 0 || para_yy % 400 == 0));
	if (para_yy < epoch || para_yy > 1999 || para_mm < 1 || para_mm > 12 ||
		para_dd < 1 || para_dd > mdays[--para_mm])
		return (-1);
	jdate = para_dd - 1;
	for (i = 0; i < para_mm; i++)
		jdate += mdays[i];
	for (i = epoch; i < para_yy; i++)
		jdate += 365 + (i % 4 == 0);
	jdate *= daysec;
	jdate += para_zone * 60L;
	if ((tod = timeconv (para_h, para_m, para_s, para_mer)) < 0)
		return (-1);
	jdate += tod;
	if (para_dayflag == DAYLIGHT || (para_dayflag == MAYBE && localtime (&jdate)->tm_isdst))
		jdate += -1 * 60 * 60;
	return (jdate);
}

time_t 
dayconv (int para_ord, int para_day, time_t para_now)
{
	register struct tm *loctime;
	time_t tod;

	tod = para_now;
	loctime = localtime (&tod);
	tod += daysec * ((para_day - loctime->tm_wday + 7) % 7);
	tod += 7 * daysec * (para_ord <= 0 ? para_ord : para_ord - 1);
	return daylcorr (tod, para_now);
}

time_t 
timeconv (int para_hh, int para_mm, int para_ss, int para_mer)
{
	if (para_mm < 0 || para_mm > 59 || para_ss < 0 || para_ss > 59)
		return (-1);
	switch (para_mer)
	{
	case AM:
		if (para_hh < 1 || para_hh > 12)
			return (-1);
		return (60L * ((para_hh % 12) * 60L + para_mm) + para_ss);
	case PM:
		if (para_hh < 1 || para_hh > 12)
			return (-1);
		return (60L * ((para_hh % 12 + 12) * 60L + para_mm) + para_ss);
	case 24:
		if (para_hh < 0 || para_hh > 23)
			return (-1);
		return (60L * (para_hh * 60L + para_mm) + para_ss);
	default:
		return (-1);
	}
}
time_t 
monthadd (time_t para_sdate, time_t para_relmonth)
{
	struct tm *ltime;
	int local_mm;
	int local_yy;

	if (para_relmonth == 0)
		return 0;
	ltime = localtime (&para_sdate);
	local_mm = 12 * ltime->tm_year + ltime->tm_mon + para_relmonth;
	local_yy = local_mm / 12;
	local_mm = local_mm % 12 + 1;
	return daylcorr (dateconv (local_mm, ltime->tm_mday, local_yy, ltime->tm_hour,
				  ltime->tm_min, ltime->tm_sec, 24, ourzone, MAYBE), para_sdate);
}

time_t 
daylcorr (time_t future, time_t now)
{
	int fdayl,
	 nowdayl;

	nowdayl = (localtime (&now)->tm_hour + 1) % 24;
	fdayl = (localtime (&future)->tm_hour + 1) % 24;
	return (future - now) + 60L * 60L * (nowdayl - fdayl);
}

static char *lptr;

int 
yylex (void)
{
	/* extern int yylval; */
	int sign;
	register char c;
	register char *p;
	char idbuf[20];
	int pcnt;

	for (;;)
	{
		while (isspace (*lptr))
			lptr++;

		if (isdigit (c = *lptr) || c == '-' || c == '+')
		{
			if (c == '-' || c == '+')
			{
				if (c == '-')
					sign = -1;
				else
					sign = 1;
				if (!isdigit (*++lptr))
				{
					/* yylval = sign; return (NUMBER); */
					return yylex ();	/* skip the '-' sign */
				}
			}
			else
				sign = 1;
			yylval = 0;
			while (isdigit (c = *lptr++))
				yylval = 10 * yylval + c - '0';
			yylval *= sign;
			lptr--;
			return (NUMBER);

		}
		else if (isalpha (c))
		{
			p = idbuf;
			while (isalpha (c = *lptr++) || c == '.')
				if (p < &idbuf[sizeof (idbuf) - 1])
					*p++ = c;
			*p = '\0';
			lptr--;
			return (lookup (idbuf));
		}

		else if (c == '(')
		{
			pcnt = 0;
			do
			{
				c = *lptr++;
				if (c == '\0')
					return (c);
				else if (c == '(')
					pcnt++;
				else if (c == ')')
					pcnt--;
			}
			while (pcnt > 0);
		}

		else
			return (*lptr++);
	}
}

struct table
{
	const char *name;
	int type,
	 value;
};

struct table mdtab[] =
{
	{"january", MONTH, 1},
	{"february", MONTH, 2},
	{"march", MONTH, 3},
	{"april", MONTH, 4},
	{"may", MONTH, 5},
	{"june", MONTH, 6},
	{"july", MONTH, 7},
	{"august", MONTH, 8},
	{"september", MONTH, 9},
	{"sept", MONTH, 9},
	{"october", MONTH, 10},
	{"november", MONTH, 11},
	{"december", MONTH, 12},

	{"sunday", DAY, 0},
	{"monday", DAY, 1},
	{"tuesday", DAY, 2},
	{"tues", DAY, 2},
	{"wednesday", DAY, 3},
	{"wednes", DAY, 3},
	{"thursday", DAY, 4},
	{"thur", DAY, 4},
	{"thurs", DAY, 4},
	{"friday", DAY, 5},
	{"saturday", DAY, 6},
	{0, 0, 0}};

#define HRS *60
#define HALFHR 30
struct table mztab[] =
{
	{"a.m.", MERIDIAN, AM},
	{"am", MERIDIAN, AM},
	{"p.m.", MERIDIAN, PM},
	{"pm", MERIDIAN, PM},
	{"nst", ZONE, 3 HRS + HALFHR},	/* Newfoundland */
	{"n.s.t.", ZONE, 3 HRS + HALFHR},
	{"ast", ZONE, 4 HRS},		/* Atlantic */
	{"a.s.t.", ZONE, 4 HRS},
	{"adt", DAYZONE, 4 HRS},
	{"a.d.t.", DAYZONE, 4 HRS},
	{"est", ZONE, 5 HRS},		/* Eastern */
	{"e.s.t.", ZONE, 5 HRS},
	{"edt", DAYZONE, 5 HRS},
	{"e.d.t.", DAYZONE, 5 HRS},
	{"cst", ZONE, 6 HRS},		/* Central */
	{"c.s.t.", ZONE, 6 HRS},
	{"cdt", DAYZONE, 6 HRS},
	{"c.d.t.", DAYZONE, 6 HRS},
	{"mst", ZONE, 7 HRS},		/* Mountain */
	{"m.s.t.", ZONE, 7 HRS},
	{"mdt", DAYZONE, 7 HRS},
	{"m.d.t.", DAYZONE, 7 HRS},
	{"pst", ZONE, 8 HRS},		/* Pacific */
	{"p.s.t.", ZONE, 8 HRS},
	{"pdt", DAYZONE, 8 HRS},
	{"p.d.t.", DAYZONE, 8 HRS},
	{"yst", ZONE, 9 HRS},		/* Yukon */
	{"y.s.t.", ZONE, 9 HRS},
	{"ydt", DAYZONE, 9 HRS},
	{"y.d.t.", DAYZONE, 9 HRS},
	{"hst", ZONE, 10 HRS},		/* Hawaii */
	{"h.s.t.", ZONE, 10 HRS},
	{"hdt", DAYZONE, 10 HRS},
	{"h.d.t.", DAYZONE, 10 HRS},

	{"gmt", ZONE, 0 HRS},
	{"g.m.t.", ZONE, 0 HRS},
	{"bst", DAYZONE, 0 HRS},	/* British Summer Time */
	{"b.s.t.", DAYZONE, 0 HRS},
	{"eet", ZONE, 0 HRS},		/* European Eastern Time */
	{"e.e.t.", ZONE, 0 HRS},
	{"eest", DAYZONE, 0 HRS},	/* European Eastern Summer Time */
	{"e.e.s.t.", DAYZONE, 0 HRS},
	{"met", ZONE, -1 HRS},		/* Middle European Time */
	{"m.e.t.", ZONE, -1 HRS},
	{"mest", DAYZONE, -1 HRS},	/* Middle European Summer Time */
	{"m.e.s.t.", DAYZONE, -1 HRS},
	{"wet", ZONE, -2 HRS},		/* Western European Time */
	{"w.e.t.", ZONE, -2 HRS},
	{"west", DAYZONE, -2 HRS},	/* Western European Summer Time */
	{"w.e.s.t.", DAYZONE, -2 HRS},

	{"jst", ZONE, -9 HRS},		/* Japan Standard Time */
	{"j.s.t.", ZONE, -9 HRS},	/* Japan Standard Time */
					/* No daylight savings time */

	{"aest", ZONE, -10 HRS},	/* Australian Eastern Time */
	{"a.e.s.t.", ZONE, -10 HRS},
	{"aesst", DAYZONE, -10 HRS},	/* Australian Eastern Summer Time */
	{"a.e.s.s.t.", DAYZONE, -10 HRS},
	{"acst", ZONE, -(9 HRS + HALFHR)},	/* Australian Central Time */
	{"a.c.s.t.", ZONE, -(9 HRS + HALFHR)},
	{"acsst", DAYZONE, -(9 HRS + HALFHR)},	/* Australian Central Summer */
	{"a.c.s.s.t.", DAYZONE, -(9 HRS + HALFHR)},
	{"awst", ZONE, -8 HRS},		/* Australian Western Time */
	{"a.w.s.t.", ZONE, -8 HRS},	/* (no daylight time there, I'm told */
	{0, 0, 0}};

struct table unittb[] =
{
	{"year", MUNIT, 12},
	{"month", MUNIT, 1},
	{"fortnight", UNIT, 14 * 24 * 60},
	{"week", UNIT, 7 * 24 * 60},
	{"day", UNIT, 1 * 24 * 60},
	{"hour", UNIT, 60},
	{"minute", UNIT, 1},
	{"min", UNIT, 1},
	{"second", SUNIT, 1},
	{"sec", SUNIT, 1},
	{0, 0, 0}};

struct table othertb[] =
{
	{"tomorrow", UNIT, 1 * 24 * 60},
	{"yesterday", UNIT, -1 * 24 * 60},
	{"today", UNIT, 0},
	{"now", UNIT, 0},
	{"last", NUMBER, -1},
	{"this", UNIT, 0},
	{"next", NUMBER, 2},
	{"first", NUMBER, 1},
	/* {"second", NUMBER, 2}, */
	{"third", NUMBER, 3},
	{"fourth", NUMBER, 4},
	{"fifth", NUMBER, 5},
	{"sixth", NUMBER, 6},
	{"seventh", NUMBER, 7},
	{"eigth", NUMBER, 8},
	{"ninth", NUMBER, 9},
	{"tenth", NUMBER, 10},
	{"eleventh", NUMBER, 11},
	{"twelfth", NUMBER, 12},
	{"ago", AGO, 1},
	{0, 0, 0}};

struct table milzone[] =
{
	{"a", ZONE, 1 HRS},
	{"b", ZONE, 2 HRS},
	{"c", ZONE, 3 HRS},
	{"d", ZONE, 4 HRS},
	{"e", ZONE, 5 HRS},
	{"f", ZONE, 6 HRS},
	{"g", ZONE, 7 HRS},
	{"h", ZONE, 8 HRS},
	{"i", ZONE, 9 HRS},
	{"k", ZONE, 10 HRS},
	{"l", ZONE, 11 HRS},
	{"m", ZONE, 12 HRS},
	{"n", ZONE, -1 HRS},
	{"o", ZONE, -2 HRS},
	{"p", ZONE, -3 HRS},
	{"q", ZONE, -4 HRS},
	{"r", ZONE, -5 HRS},
	{"s", ZONE, -6 HRS},
	{"t", ZONE, -7 HRS},
	{"u", ZONE, -8 HRS},
	{"v", ZONE, -9 HRS},
	{"w", ZONE, -10 HRS},
	{"x", ZONE, -11 HRS},
	{"y", ZONE, -12 HRS},
	{"z", ZONE, 0 HRS},
	{0, 0, 0}};

int 
lookup (const char *id)
{
#define gotit (yylval=i->value,  i->type)
#define getid for(j=idvar, k=id; (*j++ = *k++)!=0; )

	char idvar[20];
	register char *j;
	register const char *k;
	register struct table *i;
	int abbrev;

	getid;
	if (strlen (idvar) == 3)
		abbrev = 1;
	else if (strlen (idvar) == 4 && idvar[3] == '.')
	{
		abbrev = 1;
		idvar[3] = '\0';
	}
	else
		abbrev = 0;

	for (j=idvar;*j;j++)
	{
		if (isascii(*j) && isupper(*j))
			*j=tolower(*j);
	}

	for (i = mdtab; i->name; i++)
	{
		j = idvar;
		for (k = i->name; *k++ == *j++;)
		{
			if (abbrev && k == i->name + 3)
				return gotit;
			if (k[-1] == 0)
				return gotit;
		}
	}

	for (i = mztab; i->name; i++)
		if (strcmp (i->name, idvar) == 0)
			return gotit;

	for (i = unittb; i->name; i++)
		if (strcmp (i->name, idvar) == 0)
			return gotit;

	if (idvar[strlen (idvar) - 1] == 's')
		idvar[strlen (idvar) - 1] = '\0';
	for (i = unittb; i->name; i++)
		if (strcmp (i->name, idvar) == 0)
			return gotit;

	for (i = othertb; i->name; i++)
		if (strcmp (i->name, idvar) == 0)
			return gotit;

	if (strlen (idvar) == 1 && isalpha (*idvar))
	{
		for (i = milzone; i->name; i++)
			if (strcmp (i->name, idvar) == 0)
				return gotit;
	}

	return (ID);
}

time_t 
get_date (char *p, time_t now, long zone)
{
#define mcheck(f)	if (f>1) err++
	int err;
	struct tm *lt;
	time_t sdate,
	 tod;

	lptr = p;
	if (now <= 0)
		(void) time (&now);
	lt = localtime (&now);
	year = lt->tm_year;
	month = lt->tm_mon + 1;
	day = lt->tm_mday;
	relsec = 0;
	relmonth = 0;
	timeflag = zoneflag = dateflag = dayflag = relflag = 0;
	daylite = MAYBE;
	hh = mm = ss = 0;
	merid = 24;
	ourzone = zone;

	if ((err = yyparse ()) != 0)
		return (-1);

	mcheck (timeflag);
	mcheck (zoneflag);
	mcheck (dateflag);
	mcheck (dayflag);

	if (err)
		return (-1);
	if (dateflag || timeflag || dayflag)
	{
		sdate = dateconv (month, day, year, hh, mm, ss, merid, ourzone, daylite);
		if (sdate < 0)
			return -1;
	}
	else
	{
		sdate = now;
		if (relflag == 0)
			sdate -= (lt->tm_sec + lt->tm_min * 60 +
					  lt->tm_hour * (60L * 60L));
	}

	sdate += relsec;
	sdate += monthadd (sdate, relmonth);

	if (dayflag && !dateflag)
	{
		tod = dayconv (dayord, dayreq, sdate);
		sdate += tod;
	}

	return sdate;
}

int 
yyerror (const char *s)
{
	return 0;
}
