/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 1995 1996 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   errormail.c: Fehlermeldungen verschicken.


   Zustand zur Zeit: Uebel.
*/

#include <stdio.h>
#include "mausconsys.h"

#include <stdarg.h>
#include <syslog.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "metamsg.h"
#include "errormail.h"
#include "rfc.h"
#include "srfcmail.h"

#define MAXEMP	10

/* ------------------------------------------------------------------------- */


void
errormail (FILE * input, const char *subject, const char *errortext,...)
{
	va_list ap;
	int zahl;
	int i;
	int end = 0;
	char *emp[MAXEMP];
	FILE *f[MAXEMP];
	char buf[1024];
	BOOLEAN twice=FALSE;

	zahl = 0;

	va_start (ap, errortext);

	if (0!=strcmp("translate this",_("translate this")))
		twice=TRUE;

	while (zahl < MAXEMP && !end)
	{
		emp[zahl] = va_arg (ap, char *);

		end = (emp[zahl] == NULL);
		if (!end)
			zahl++;
	}
	va_end (ap);

	for (i = 0; i < zahl; i++)
	{
		char s[128];

		sprintf (s, "mail -s '%s' %s", subject, emp[i] + 1);
		f[i] = popen (s, "w");
		if (!f[i])
		{
			int e;

			e = errno;
			fprintf (stderr,_(
				"Fatal Mauscon error: could not send error messages\n"
				"Message lost due to error:\n"
				"Subject of error message: %s\n"
				"\n"
				"%s\n"),
					 subject, errortext);
			log (LOG_CRIT, _("Message lost, couldn't send error message"));
			log (LOG_CRIT, _("could not popen to mail: %s"), strerror (e));
		}
	}
	for (i = 0; i < zahl; i++)
	{
		if (f[i])
		{
			const char *str=N_(
				"MausTauschgateway %s <-> %s (Mauscon version %s)\n"
				"\n"
				"Critical problem during delivery:\n"
				"%s\n"
				"%s\n");
			const char *str2=N_("failed message follows");

			fprintf (f[i], str,
					 opts.pollsite,opts.mysitedomain, VERSION,
					 subject, errortext);
			if (twice) {
				fprintf (f[i], _(str),
						 opts.pollsite,opts.mysitedomain, VERSION,
						 subject, errortext);
			}
			if (emp[i][0] == '-' || emp[i][0] == '=')
				dump_conf (f[i]);
			if (emp[i][0] == '+' || emp[i][0] == '=') {
				fputs ("------ ",stderr);
				fputs (str2,stderr);
				if (twice) {
				fputs (" - ",stderr);
				fputs (_(str2),stderr);
				}
				fputs ("------\n",stderr);
			}
		}
	}
	if (input)
		while (fgets (buf, sizeof (buf) - 1, input))
		{
			for (i = 0; i < zahl; i++)
				if (f[i])
					fprintf (f[i], "%s\n", buf);
		}
	else
	{
		for (i = 0; i < zahl; i++)
			if (f[i])
				fprintf (f[i], _("message completely lost!\n"));
	}
	for (i = 0; i < zahl; i++)
		if (f[i])
			pclose (f[i]);
}


void
bounce (const char *to, const char *reason, const char *mbuf)
{
	char buf[1024];

	FILE *pipef;

	sprintf (buf, "mail -s 'message failed' '%s'", to);
	pipef = popen (buf, "w");
	if (!pipef)
	{
		char g[1024];
		char h[PATH_MAX * 2];
		int e;

		e = errno;
		sprintf (g, "=%s", opts.gatemaster);
		sprintf (h, "%s: %s", buf, strerror (errno));
		errormail (NULL, "error bouncing message",
				   h, g, NULL);
		log (LOG_CRIT, _("could not bounce message: %s"),
			 strerror (e));
		return;
	}
	fprintf (pipef, "%s\n", reason);
	if (mbuf)
	{
		fprintf(pipef,_("failed message follows:\n"));
		fputs(mbuf,pipef);
	}
	else
		fprintf(pipef,_("failed message completely lost\n"));
	pclose (pipef);
}

void
new_bounce (const char *to, 
			metamsg_t *orig_msg,
			const char *mbuf,
			struct bounce_info *info)
{
	FILE *f;
	char buf[1024];
	char boundary[1024];
	static int nummer=0;
	struct tm *tm;
	time_t t;
	metamsg_t *meta;

	if (!opts.enable_dsn_bounces && mbuf) {
		bounce(to, 
			   info->diagnostic_code ? info->diagnostic_code 
			   : "no reason given",
			   mbuf);
		return;
	}

	meta=meta_alloc();
	XNULLCHECK(meta);
	sprintf(buf,"mauscon.bounce.%ld.%ld.%d@%s",
			(long) time(NULL), (long) getpid(), ++nummer,
			opts.mysitedomain);
	meta->message_id=XSTRDUP(buf);
	meta->env_to=XSTRDUP(to);
	meta->env_from=NULL; /* smtp code for "don't bounce this */

	f=tmpfile();

	if (!f)
	{
		char g[1024];
		char h[PATH_MAX * 2];
		int e;
		
		e = errno;
		sprintf (g, "=%s", opts.gatemaster);
		sprintf (h, "%s: %s", buf, strerror (errno));
		errormail (NULL, "error bouncing message",
				   h, g, NULL);
		log (LOG_CRIT, _("could not bounce message: %s"),
			 strerror (e));
		return;
	}

	fprintf(f,"Message-ID: <%s>\r\n",meta->message_id);
	fprintf(f,"From: Mauscon@%s\r\n",opts.mysitedomain);
	fprintf(f,"To: %s\r\n",to);
	fprintf(f,"Subject: message delivery error\r\n");
	if (info->orig_id) 
	{
		if (info->orig_id[0]=='<')
			fprintf(f,"In-Reply-To: %s\r\n",info->orig_id);
		else
			fprintf(f,"In-Reply-To: <%s>\r\n",info->orig_id);
	}

	time(&t);
	tm=localtime(&t);
	rfc_make_date (buf, sizeof(buf),tm);
	fprintf(f,"Date: %s\r\n",buf);

	fprintf(f,"Mime-Version: 1.0\r\n");
	sprintf(boundary,"mauscon.bounce.%ld.%ld.%d.%s",
			(long) time(NULL), (long) getpid(), ++nummer,
			opts.mysitedomain);
	fprintf(f, 
			"Content-Type: multipart/report; report-type=delivery-status;\r\n"
			"\tboundary=%s\r\n", 
			boundary);
	fprintf(f,"\r\n");

	/* part zero ... for those plagued by pine and other, who cannot
	 * handle multipart/report */
	fprintf(f,
			"The following message:\r\n"
			"\r\n"
			"Original-Message-ID: %s\r\n",
			info->orig_id ? info->orig_id : "not available");
	if (info->orig_subject)
		fprintf(f,"Original-Subject: %s\r\n",info->orig_subject);
	else if (orig_msg && orig_msg->subject)
		fprintf(f,"Original-Subject: %s\r\n",orig_msg->subject);
	fprintf(f,
		"\r\n"
		"to %s\r\n"
		"\r\n"
		"has not been delivered. Reason:\r\n"
		"%s\r\n"
		"\r\n",
		info->orig_to ? info->orig_to : "an unknown recipient",
		info->diagnostic_code ? info->diagnostic_code : "not available"
		);

	/* first body part, for the human reader */
	fprintf(f,"--%s\r\n",boundary);
	fprintf(f,
			"Content-Type: text/plain\r\n"
			"\r\n"
			"The following message:\r\n"
			"\r\n"
			"Original-Message-ID: %s\r\n",
			info->orig_id ? info->orig_id : "not available");
	if (info->orig_subject)
		fprintf(f,"Original-Subject: %s\r\n",info->orig_subject);
	else if (orig_msg && orig_msg->subject)
		fprintf(f,"Original-Subject: %s\r\n",orig_msg->subject);
	fprintf(f,
		"\r\n"
		"to %s\r\n"
		"\r\n"
		"has not been delivered\r\n"
		"\r\n",
		info->orig_to ? info->orig_to : "an unknown recipient"
		);

	/* second part. machine readable */
	fprintf(f,"--%s\r\n",boundary);
	fprintf(f,
		"content-type: message/delivery-status\r\n"
		"\r\n"
		"reporting-mda: dns;%s\r\n",
		opts.mysitedomain
		);
	fprintf(f,
		"Final-Recipient: rfc822;%s\r\n"
		"Action: failed\r\n"
		"Status: %s\r\n",
		info->orig_to ? info->orig_to : "unknown",
		info->status_code
		);
	if (info->remote_mta) 
		fprintf(f,"Remote-nta: %s\r\n",info->remote_mta);
	if (info->diagnostic_code)
		fprintf(f,"Diagnostic-Code: %s\r\n",info->diagnostic_code);
	fprintf(f,"\r\n");

	/* third part. headers of message */
	/* we prefer mbuf because it is a rfc822 message */
	if (mbuf) {
		fprintf(f,"--%s\r\n",boundary);
		fprintf(f,
				"content-type: message/rfc822\r\n"
				"\r\n");
		
		fputs(mbuf,f);
	} else if (orig_msg) {
		fprintf(f,"--%s\r\n",boundary);
		fprintf(f,
				"content-type: message/rfc822\r\n"
				"X-notice: message header reconstructed by gateway\r\n"
				"\r\n"
			);
		if (orig_msg->message_id) 
			fprintf(f,"Message-ID: <%s>\r\n",orig_msg->message_id);
		if (orig_msg->subject) 
			fprintf(f,"Subject: <%s>\r\n",orig_msg->subject);
		if (orig_msg->env_to)
			fprintf(f,"Envelope-To: <%s>\r\n",orig_msg->env_to);
		if (orig_msg->head_to) {
			metalist_t *l;
			for (l=orig_msg->head_to;l;l=l->next) {
				if (l==orig_msg->head_to)
					fprintf(f,"To: %s",l->text);
				else
					fprintf(f,"\t%s",l->text);
				if (l->next)
					fprintf(f,",\r\n");
				else
					fprintf(f,"\r\n");
			}
		}
		if (orig_msg->head_from) {
			metalist_t *l;
			for (l=orig_msg->head_from;l;l=l->next) {
				if (l==orig_msg->head_from)
					fprintf(f,"From: %s",l->text);
				else
					fprintf(f,"\t%s",l->text);
				if (l->next)
					fprintf(f,",\r\n");
				else
					fprintf(f,"\r\n");
			}
		}
		if (orig_msg->sender) {
			metalist_t *l;
			for (l=orig_msg->head_from;l;l=l->next) {
				if (l==orig_msg->sender)
					fprintf(f,"Sender: %s",l->text);
				else
					fprintf(f,"\t%s",l->text);
				if (l->next)
					fprintf(f,",\r\n");
				else
					fprintf(f,"\r\n");
			}
		}
		if (orig_msg->reply_to) {
			metalist_t *l;
			for (l=orig_msg->reply_to;l;l=l->next) {
				if (l==orig_msg->sender)
					fprintf(f,"Reply-To: %s",l->text);
				else
					fprintf(f,"\t%s",l->text);
				if (l->next)
					fprintf(f,",\r\n");
				else
					fprintf(f,"\r\n");
			}
		}
		fputs(orig_msg->body.start,f);
	}
	fprintf(f,"--%s--\r\n",boundary);

	if (fflush(f)) {
		char g[1024];
		char h[PATH_MAX * 2];
		int e;

		e = errno;
		sprintf (g, "=%s", opts.gatemaster);
		sprintf (h, "%s: %s", buf, strerror (errno));
		errormail (NULL, "error bouncing message",
				   h, g, NULL);
		log (LOG_CRIT, _("could not bounce message: %s"),
			 strerror (e));
		fclose(f);
		meta_free(meta);
		return;
	}
	rewind(f);
	meta->file=f;
	if (!srfcmail(FALSE,meta)) {
		char g[1024];
		char h[PATH_MAX * 2];
		int e;

		e = errno;
		sprintf (g, "=%s", opts.gatemaster);
		sprintf (h, "%s: %s", buf, strerror (errno));
		errormail (NULL, "error bouncing message",
				   h, g, NULL);
		log (LOG_CRIT, _("could not bounce message: %s"),
			 strerror (e));
	}
	fclose(f);
	meta_free(meta);
}
