/* mauscon: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 1995 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   getmausval - gibt den Inhalt einer Config-Var auf dem stdout aus 
*/

#include "mausconsys.h"
#include <stdio.h>
#include <syslog.h>
#include <getopt.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "long-options.h"

static void
usage(int exitcode, const char *what)
{
    FILE *f=stdout;

	if (exitcode)
	{
		if (what)
			fprintf(stderr, "%s: %s\n",program_name,what);
		fprintf (stderr, _("Try `%s --help' for more information.\n"),
				 program_name);
		exit(exitcode);
	}
	
	fprintf(f,_("Usage: %s [options] channel\n"), program_name);
	fputs(_("Get configuration variables for `channel'\n"),f);
	fputs(_(
"  -c  --cnf-file FILE         use config file FILE\n"
"      --help                  this text\n"
"  -V  --variable VAR          get variable VAR\n"
"      --version               show version info and exit\n"
"\n"
"short options use the same arguments as the long ones\n"
"if no VAR is given all variables are printed out\n"
		),f);
	exit(exitcode);
}

static void
usage1(int ecode)
{
	usage(ecode,NULL);
}

int
main (int argc, char **argv)
{
	const char *cnffname=SYSCONFDIR "/" CNFNAME;
	const char *argument_value=NULL;
	struct option cmdlinopts[] =
	{
		{"val", required_argument, NULL, 'V'},
		{"variable", required_argument, NULL, 'V'},
		{"var", required_argument, NULL, 'V'},
		{"cnffile", required_argument, NULL, 'c'},
		{"help", 0, NULL, 'h'},
		{"version", 0, NULL, 'v'},
		{NULL, 0, NULL, 0}
	};
	int option_index=0;
	const char *slash;

	program_name=argv[0];
	slash=strrchr(program_name,'/');
	if (slash)
		program_name=slash+1;

    parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage1);

    while (1)
    {
        int c;

        c = getopt_long (argc, argv, "c:V:",
                         cmdlinopts, &option_index);

        /* Detect the end of the options.   */
        if (c == -1)
            break;
        switch (c)
        {
        case 'V':
            argument_value = optarg;
            break;
        case 'c':
            cnffname = optarg;
            break;
		default:
        case '?':
            log (LOG_ERR, _("unknown option %c"), c);
            usage (2,NULL);
            break;
        }
	}

    if (!argv[optind])
    {
        log (LOG_ERR, _("channel not given"));
        exit(2);
    }
    else if (argv[optind+1])
    {
        log (LOG_ERR, _("to many arguments"));
        exit (2);
    }

	do_conf(cnffname,argv[optind]);

	openlog (program_name, 0, opts.sl_facility);
	setlogmask(LOG_UPTO(opts.sl_priority));

	if (!argument_value)
		dump_conf(stdout);
	else
		printf ("%s\n", get_conf_value (argument_value));

	closelog ();

	exit (0);
}
