/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 1995 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   log.c: Logeintragungen schreiben.
*/

#include "mausconsys.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <limits.h>
#include <syslog.h>

#include "mauscon.h"
#include "mausconlib.h"

static FILE *msglogfile;

void
open_msglog(void)
{
	char fname[PATH_MAX];
	time_t t;
	struct tm *tm;
	time(&t);
	tm=localtime(&t);
	sprintf(fname,"%s/%s/msglog/",opts.spoolbase,action);
	strftime(fname+strlen(fname),PATH_MAX-strlen(fname),
		"%y_%m_%d",tm);
	msglogfile=fopen(fname,"a");
	if (!msglogfile && errno==ENOTDIR)
	{
		sprintf(fname,"%s/%s/msglog",opts.spoolbase,action);
		msglogfile=fopen(fname,"a");
	}
	if (!msglogfile)
	{
		log(LOG_ERR,_("cannot open msglog: %s"), strerror(errno));
		exit(1);
	}
}

void 
close_msglog(void)
{
	if (msglogfile)
	{
		fclose(msglogfile);
		msglogfile=NULL;
	}
}

void
msglog(const char *format,...)
{
	va_list ap;
	time_t t;
	struct tm *tm;
	FILE *out;

	static char str[32];
	static time_t last_time=0;

	if (!msglogfile)
		out = stderr;
	else
		out = msglogfile;
	va_start (ap, format);

	t = time (NULL);
	if (t!=last_time)
	{
		tm = localtime (&t);
		strftime (str, sizeof (str) - 1, "%d.%m.%y %R: ", tm);
		last_time=t;
	}
	fputs(str,out);
	vfprintf (out, format, ap);
	fputc('\n',out);
	fflush (out);
	va_end (ap);
}

void
log (int priority, const char *format,...)
{
	va_list ap;

	va_start (ap, format);
	vsyslog (priority, format, ap);
	va_end (ap);

	/* Kopie der Nachricht nach stderr. Eventuell mal konfigurierbar machen */
	va_start (ap, format);
	vfprintf (stderr, format, ap);
	fputc('\n',stderr);
	va_end (ap);
}
