/* mauscon.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#if !defined(LIB_H) || defined(INIT)
#ifndef INIT
#define LIB_H
extern const char *gate_version;

#include "config.h"

extern const char *program_name;

typedef struct
{
	char *start;
	char *next;
	size_t size; /* Nicht Laenge (next-start), sondern Puffergroesse */
} buf_t;
#define ADD_TO_BUF(buf,str) do { \
	size_t l=strlen((str)); if (l+2+(buf)->next>=(buf)->start+(buf)->size) { \
		size_t off; size_t l2=(buf)->size; if (!l2) l2=1024; \
		while(l2<l) l2*=2; (buf)->size+=l2; off=(buf)->next-(buf)->start; \
		(buf)->start=XREALLOC((buf)->start,(buf)->size); \
		(buf)->next=(buf)->start+off; \
	} \
	memcpy((buf)->next,(str),l+1); (buf)->next+=l; \
} while(0)

	/*
	 * Zeilenende fuer MausTausch, SMTP, NNTP
	 */
#define LINEEND "\r\n"
	/*
	 * Erlaubte Zeilenendezeichen. *Nach* diesen Zeichen kommen nur
	 * noch andere Zeilenendezeichen.
	 */
typedef struct mc_list_struct
{
	char *data;
	char *ext;
	struct mc_list_struct *next;
} mc_list_t;

typedef enum 
{ 
	MAUS2RFC=1, 
	RFC2MAUS=2, 
	MROUTE=4,
	GATECONFTEST=8,
	CALL=16,
	GETMAUSVAL=32,
	MCOM=64,
	MHIST=128
} PROGRAM;


/* Defaultbezeichner */
#define CNFNAME "mauscon.cnf"

/* allgemeine Libraryfunktionen */
char *strtrim(char *s);
int strhalfcasecmp(const char *uplow, const char *low);
int strnhalfcasecmp(const char *uplow, const char *low, size_t n);
int parse_bool(char *s);
FILE *xfopen(const char *fname, const char *mode);

int do_conf(const char *progname, const char *aktion);
const char *get_conf_value(const char *which);
void dump_conf(FILE *f);
time_t get_date(char *p, time_t now, long zone);

int is_allowed(const char *);

typedef struct cancel_struct
{
	char *id;
	char *from;
	struct cancel_struct *next;
	int done;
} cancel_t;

/* canceln von Nachrichten */
int cancel_init(int keep_locked);
cancel_t *cancel_find_in_list(const char *id);
cancel_t *cancel_anker(void);
void cancel_end(int delete,int rewrite);
void cancel_add(const char *id, const char *from);

/* Libraryfunktionen fuer Logfile */
void open_msglog(void);
void close_msglog(void);
void log(int priority, const char *format, ...)
     MC_ATTRIB_PRINTF(2,3);
void msglog(const char *format, ...)
     MC_ATTRIB_PRINTF(1,2);


/* Kommandozeilenauswertung */
extern const char *action;
enum work_only_enum { ONLY_ALL, ONLY_MAIL, ONLY_NEWS, ONLY_SPECIAL };
extern int work_only;
extern int dont_use_nntp;
extern int dont_use_smtp;

typedef enum { USER, GATE } WORKMODE;
typedef enum { LOGMODE_BOTH, LOGMODE_SYSLOG, LOGMODE_FILE } LOGMODE;
typedef enum 
{ 
	BOXTYP_NONE,
	BOXTYP_QUARK, 
	BOXTYP_MAUS 
} BOXTYP;

typedef enum 
{ 
	REKILL_FULL,  /* kill all re: */
	REKILL_SMART, /* kill re: if line too long */
	REKILL_NONE   /* don't kill re: */
} REKILLMODE;

/***************************************************************************
 * Variablen der Konfigurationsdatei
 * genutzt fuer Deklaration und Initialisierung, via INIT-Makro 
 ***************************************************************************/
#define INIT(x,y)	x;
#define IS_DECLARE
extern struct overalloptions {
#else
struct overalloptions opts= {
#endif
	INIT(WORKMODE workmode,                USER)

	INIT(const char *spoolbase,         "/var/spool/mauscon")
	INIT(const char *tempdir,           "/var/tmp")

	INIT(int sl_facility,               LOG_NEWS)
	INIT(int sl_priority,               LOG_NOTICE)
	INIT(int smart_LOG,                 0)

	INIT(const char *gatemaster,        "postmaster")
	INIT(const char *mysitedomain,      NULL)
	INIT(const char *mailuserdomain,       NULL)
	INIT(mc_list_t *allowed,            NULL)

	INIT(BOXTYP boxtyp,                 BOXTYP_MAUS)
	INIT(const char *pollsite,             NULL)
	INIT(const char *pollsitedomain,       NULL)
	INIT(const char *mausdomain,        ".maus.de")
	INIT(short tabsize,                 4)
	INIT(long split_after,              15500)
	INIT(short subjectlen,              250)
	INIT(int autoinfosubscribe,         1)
	INIT(int rekillmode,                REKILL_FULL)
	INIT(int send_crosspostings,        0)
	INIT(int send_sender,               0)
	INIT(int send_followup_to,          0)
	INIT(int send_reply_to,             0)
	INIT(int send_path,                 0)
	INIT(int generate_empty_mail_body,  1)
	INIT(int cancel_original,           1)
	INIT(int mail_status_change,           1)
	INIT(char autostatus,              '\0')
	INIT(int strip_domain,              1)
	INIT(int strip_bangpath,            0)
	INIT(int t_online_id_hack,          0)


	INIT(int direct_smtp,                  0)
	INIT(const char *mailhost,             "localhost")
	INIT(const char *reroute_host,      NULL)
	INIT(const char *newshost,             "localhost")
	INIT(const char *path_entry,        "MausNet")
	INIT(const char *gprefix,           "maus")
	INIT(const char *usenet_dist_lokal,    "lokal")
	INIT(const char *usenet_dist_mausnet,  "mausnet")
	INIT(int warn_news_routing,         0)
	INIT(int unlink_unallowed,          1)
	INIT(short max_references,          4)
	INIT(int enable_dsn_bounces,        0)

	INIT(int foreign_export_kill, 0)
	INIT(const char *foreign_export_path, NULL)
	INIT(const char *foreign_export_distribution, NULL)
	INIT(int unconvertable_from_kill, 0)
	INIT(const char *unconvertable_from_path, NULL)
	INIT(const char *unconvertable_from_distribution, NULL)

	INIT(int mime_coded_headers,        0)
	INIT(int mime_coded_body,           0)
	INIT(int mime_headerlines,          1)

	INIT(const char *rmail_route,       NULL)
	INIT(const char *rsmtp_route,       NULL)
	INIT(const char *rnews_route,       NULL)

	INIT(const char *compress,              "gzip")
	INIT(short lock_tries,              10)
	INIT(short lock_sleep,              5)
	INIT(const char *diffcmd,           "diff -u --report-identical-files")
	INIT(int keep_sent,                 0)

#ifdef IS_DECLARE
#undef INIT
#undef IS_DECLARE
	char *dummy;
} opts;
#else
	NULL
};
#endif

#endif
