
/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 1995 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   mroute.c: Spoolt News+Mail Richtung MausNet
*/

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <getopt.h>
#include <limits.h>
#include <syslog.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "metamsg.h"
#include "errormail.h"
#include "rfc.h"
#include "route.h"
#include "long-options.h"

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif

static char error_to[1024];
static char gm_addr[256];
static int is_routing=0;
static int got_cancels=0;

/* two dummies, linking srfcmail/srfcnews needs them */
int dont_use_nntp;
int dont_use_smtp;

struct empfaenger {
	char *adr;
	struct empfaenger *next;
};

static int argument_test=0;

static long get_transfer_id (void);
static int handle_mail(const char *inputfilename, const  char *from, const char *to);
static int handle_news(const char *inputfilename);
static int handle_rnews(const char *inputfilename);
static int handle_rsmtp(const char *inputfilename);
static int handle_rmail(const char *inputfilename, char **adressen);
static void	test_routing(void);
static void finish_rsmtp(char *from, struct empfaenger *emp, buf_t *message);
static void cleanup_empfaenger(struct empfaenger **anker);
static void usage(int, const char *);
static void usage1(int);


buf_t mbuf;

static int
work_cancel(char *s, char *from)
{
	char *p;
	while (isspace(*s))
		s++;
	while (s && *s)
	{
		p=strchr(s,' ');
		if (*s!='<')
			return 1; /* das ist keine ID */
		if (p)
		{
			if (p[-1]=='>')
				p[-1]='\0';
			else
				return 1;
		}
		else
		{
			char *p2;
			p2=s+strlen(s)-1;
			if (*p2!='>')
				return 1;
			*p2='\0';
		}
		if (got_cancels++==0)
		{
			cancel_init(1);
		}
		cancel_add(s+1,from);
		s=p;
		if (s)
		{
			while (isspace(*s))
				s++;
		}
	} /* while (s && *s) */
	return 0;
}  /* work cancel */

static long
get_transfer_id (void)
{
	int count;
	int err;
	long mid;
	char midname[2 * PATH_MAX];
	char lockname[2 * PATH_MAX];
	char str[255];
	FILE *f;

	sprintf (midname, "%s/%s/transfer.id", opts.spoolbase, action);
	sprintf (lockname, "%s.LCK", midname);
	count = 0;
	err = 0;
	while (1)
	{
		if (!lock_create(lockname,NULL))
			break;
		count++;
		if (opts.lock_tries && count>=opts.lock_tries)
		{
			errormail (NULL,
				error_to ?  "error spooling message"
				:	"error spooling message - sender unknown",
		   		"couldn't get a transfer id (spooling message failed)",
		   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
		if (opts.lock_sleep > 0)
			sleep (opts.lock_sleep);
	}
	f = fopen (midname, "r+");
	if (!f)
	{
		/* einfach waere es, jetzt 0 zurueckliefern. Haken dabei:
		   Falls die Datei mal irgendwie "verschwindet", wuerden neue
		   Nachrichten unter den alten IDs erzeugt, und zumindest der
		   Dupecheck der Quark II findet das nicht so toll.
		   Also nehmen wir einen von der aktuellen Uhrzeit bzw. Datum
		   abhaengigen Wert. Problem: User-IDs in MAUS duerfen nur
		   bis zu 10 Zeichen lang sein. Da der Prefix "mail" vor
		   Mails aber schon vier Zeichen belegt, kann der 32-Bit-Wert
		   von time() nicht direkt verwendet werden. Nimmt man aber den
		   Rest einer Division des time()-Rueckgabewerts durch eine
		   Million, sollten Kollisionen hinreichend unwahrscheinlich
		   sein.
		*/
		mid=time(NULL) % 1000000;
		f=fopen(midname,"w");
		if (!f)
		{
			errormail (NULL,
				error_to ?
			   		"error spooling message"
				:	"error spooling message - sender unknown",
			   "couldn't write transfer id (spooling message failed)",
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	else
	{
		fgets (str, sizeof (str) - 1, f);
		mid = atol (str) + 1;
		rewind (f);
	}
	fprintf (f, "%ld", mid);
	fclose (f);
	lock_delete(lockname);
	return mid;
}

static int 
handle_rmail(const char *inputfilename, char **adressen)
{
	/* 
	 * rmail empfnger1 empfnger2 ... 
	 * Anders gesagt: Wir haben keinen Absender, und nur die Defaultroute.
	 */
	int fd;
	int i;
	long id;
	char fname[PATH_MAX];
	char env_buf[2048];
	const char *msgid=NULL;
	const char *from=NULL;

	/* 
	 * Nun die Nachricht lesen 
	 */
	if (inputfilename)
	{
		/* Dann Nachricht aus der Datei lesen */
		mbuf.next=mbuf.start;
		if (rfc_read_message(inputfilename,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	else
	{
		/* Dann Nachricht von stdin lesen */
		mbuf.next=mbuf.start;
		if (rfc_read_message_fd(0,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,_("stdin: %s\n"), strerror (errno));
			errormail (NULL,
			   "weird: cannot read stdin",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	from=rfc_find_header(mbuf.start,"from");
	if (!from)
		from=rfc_find_header(mbuf.start,"sender");
	if (!from)
		from=rfc_find_header(mbuf.start,"reply-to");
	if (!from)
		from="[mroute: unknown sender]";

	if (!msgid)
   		msgid=rfc_find_header(mbuf.start,"message-id");
   	if (!msgid)
   		msgid="[mroute/rmail: unknown id]";

   	while (*adressen)
   	{

		i=0;
		do
		{
			/* 
			 * eine Transfer-ID holen 
			 */
			id = get_transfer_id ();
			/* 
			 * In welches File spoolen? 
			 */
			sprintf (fname, "%s/%s/outgoing/mail%ld", 
				opts.spoolbase, action, (long) id);
			fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
		} while (i<100 && fd<0);

		if (fd<0)
		{
			/* Mail immer bouncen */
#define REASON "cannot open spool file, & out of memory"
			char *reason=concat("Cannot open spool file for mail to ",
				*adressen,NULL);
			bounce(from,reason ? reason : REASON , mbuf.start);
			msglog("# %s: %s, bounced",msgid,reason ? reason : REASON );
			errormail (NULL,
				reason ? reason : REASON,
			   strerror (errno),
			   gm_addr, NULL);
			adressen++;
			free(reason);
			continue;
#undef REASON
		}

		{
			char *p;
			/* uucp-Stil-Adressen "korrigieren". Die verursachen mehr Schaden 
			 * als Nutzen 
			 */
			if (!strchr(*adressen,'@') && (p=strchr(*adressen,'!'))!=NULL )
			{
				char *tmp=XMALLOC(strlen(*adressen)+1);
				*p++=0;
				sprintf(tmp,"%s@%s",p,*adressen);
				*adressen=tmp; /* slight memory leck */
			}
			if (!strchr(from,'@') && (p=strchr(from,'!'))!=NULL )
			{
				char *tmp=XMALLOC(strlen(from)+1);
				*p++=0;
				sprintf(tmp,"%s@%s",p,from);
				from=tmp; /* slight memory leck */
			}
			sprintf(env_buf,
				"X-MausConnect-Env-To: %s\n"
				"X-MausConnect-Env-From: %s\n",
				*adressen,
				from);
		}

		if (xwrite(fd,env_buf,strlen(env_buf))<0 
			|| xwrite(fd,mbuf.start,strlen(mbuf.start))<0)
		{
			const char *reason2=
				"cannot write to spool file, & out of memory";
			char *reason=concat("Cannot write to spool file for mail to ",
				*adressen,NULL);
			close(fd);
			unlink(fname);
			/* Mail immer bouncen */
			bounce(from, reason ? reason : reason2, mbuf.start);
			errormail (NULL,
				reason ? reason : reason2,
			   strerror (errno),
			   gm_addr, NULL);
			msglog("# %s: %s, bounced",msgid, reason ? reason : reason2);
			free(reason);
#undef REASON
		}
		else
			msglog("# %s: rmail spooled as %ld",msgid,id);
		close(fd);
		adressen++;
	}
	exit(0);
}

static int 
handle_mail(const char *inputfilename, const char *from, const char *to)
{
	int fd;
	int i;
	long id;
	char fname[PATH_MAX];
	char env_buf[2048];
	const char *msgid=NULL;
	char *route;

	if (!to)
	{
		/* we should never be here? */
		log(LOG_ERR,_("receiver of mail not given"));
		exit(1);
	}
	if (from)
	{
		strcpy(error_to,from);
	}
	else
	{
		log(LOG_ERR,_("sender of mail not given"));
		exit(1);
	}
	if (argument_test)
	{
		route=route_it(from,to);
		if (route)
			printf(_("route for %s to %s via %s\n"),from, to,route);
		else
			printf(_("no route for %s to %s\n"),from,to);
		return 0;
	}

	/* 
	 * Nun die Nachricht lesen 
	 */
	if (inputfilename)
	{
		/* Dann Nachricht aus der Datei lesen */
		mbuf.next=mbuf.start;
		if (rfc_read_message(inputfilename,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}
	else
	{
		/* Dann Nachricht von stdin lesen */
		mbuf.next=mbuf.start;
		if (rfc_read_message_fd(0,&mbuf))
		{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"stdin: %s\n", strerror (errno));
			errormail (NULL,
			   "weird: cannot read stdin",
			   strerror (errno),
			   gm_addr, *error_to ? (error_to) : (NULL) , NULL);
			exit (1);
		}
	}

	if (!msgid)
   		msgid=rfc_find_header(mbuf.start,"message-id");
   	if (!msgid)
   		msgid="[mroute: unknown id]";

	/* 
	 * Die Nachricht ist jetzt also im Puffer. Nun routen.
	 */
	if (is_routing)
	{
		route=route_it(from,to);
		if (!route)
		{
			msglog("# %s: no route %s -> %s",
				msgid,from,to);
		}
		if (!route && opts.reroute_host)
		{
			/* Mail via eine andere Kiste schicken */
			char adr[1024];
			char cmd[2048];
			char *p;
			FILE *wfd;
			strcpy(adr,to);
			p=strchr(adr,'@');
			if (p)
				*p='%';
			strcat(adr,"@");
			strcat(adr,opts.reroute_host);
			sprintf(cmd,"rmail '%s'",adr);
			wfd=popen(cmd,"w");
			if (wfd)
			{
				fwrite(mbuf.start,strlen(mbuf.start),1,wfd);
				msglog("! %s: rerouted via %s",msgid,opts.reroute_host);
				exit(0);
			}
			msglog("# %s: rerouting failed with %s",msgid,strerror(errno));
		}
		if (!route)
		{
			struct bounce_info b;
			memset(&b,0,sizeof(b));
			b.orig_id=msgid;
			b.orig_subject=rfc_find_header(mbuf.start,"subject");
			b.orig_to=to;
			b.status_code="5.7.1"; /* Delivery not authorized, message refused */
				/* b.remote_mta=.. keiner beteiligt */
			b.diagnostic_code=concat("x-local; no route from ",
									 from," to ", to, NULL);
			XNULLCHECK(b.diagnostic_code);
			new_bounce(from,NULL,mbuf.start,&b);
			msglog("# %s: bounced",msgid);
			exit(0);
		}
		action=route;
	} else if (!is_allowed(from)) {
		struct bounce_info b;
		memset(&b,0,sizeof(b));
		b.orig_id=msgid;
		b.orig_subject=rfc_find_header(mbuf.start,"subject");
		b.orig_to=to;
		b.status_code="5.7.1"; /* Delivery not authorized, message refused */
		/* b.remote_mta=.. keiner beteiligt */
		b.diagnostic_code=concat("x-local; ", from,
								 " is not allowed to use the gateway", NULL);
		XNULLCHECK(b.diagnostic_code);
		new_bounce(from,NULL,mbuf.start,&b);
		msglog("# %s: bounced",msgid);
		exit(0);
	}

	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/mail%ld", 
			opts.spoolbase, action, (long) id);
		fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
	} while (i<100 && fd<0);


	if (fd<0)
	{
		/* Mail immer bouncen */
		bounce(from,"Cannot open spool file",
			mbuf.start);
		msglog("# %s: cannot open spoolfile, bounced",msgid);
		errormail (NULL,
		   "weird: cannot open spool file",
		   strerror (errno),
		   gm_addr, NULL);
		exit(0);
	}

	{
		char *p;
		/* uucp-Stil-Adressen "korrigieren". Die verursachen mehr Schaden 
		 * als Nutzen 
		 */
		if (!strchr(to,'@') && (p=strchr(to,'!'))!=NULL )
		{
			char *tmp=XMALLOC(strlen(to)+1);
			*p++=0;
			sprintf(tmp,"%s@%s",p,to);
			to=tmp; /* slight memory leck */
		}
		if (!strchr(from,'@') && (p=strchr(from,'!'))!=NULL )
		{
			char *tmp=XMALLOC(strlen(from)+1);
			*p++=0;
			sprintf(tmp,"%s@%s",p,from);
			from=tmp; /* slight memory leck */
		}
		sprintf(env_buf,
			"X-MausConnect-Env-To: %s\n"
			"X-MausConnect-Env-From: %s\n",
			to,
			from);
	}

	if (xwrite(fd,env_buf,strlen(env_buf))<0 
		|| xwrite(fd,mbuf.start,strlen(mbuf.start))<0)
	{
		const char *es=strerror(errno);
		close(fd);
		unlink(fname);
		/* Mail immer bouncen */
		bounce(from,"Cannot spool message",
			mbuf.start);
		errormail (NULL,
		   "weird: cannot spool mail",
		   es,
		   gm_addr, NULL);
		msglog("# %s: cannot spool messaged, bounced: %s",msgid,es);
		exit(0);
	}

	msglog("# %s: mail spooled as %ld",msgid,id);
	close(fd);
	exit(0);
}

static int 
handle_news(const char *inputfilename)
{
	char *from;
	metalist_t *ml;
	char *route;
	const char *msgid;
	long id;
	char fname[PATH_MAX];
	int ok;
	int fd;
	int i;

	/* Bei News mssen alle wesentlichen Daten aus der Nachricht entnommen
	 * werden.
	 */
	if (inputfilename == NULL)
	{
		mbuf.next=mbuf.start;
    	if (rfc_read_message_fd(0,&mbuf))
    	{
			/* Hm? Eingabedatei nicht da? */
			const char *es=strerror(errno);
			log(LOG_ERR,_("stdin: %s\n"), es);
			errormail (NULL,
			   "weird: cannot read stdin",
			   es,
			   gm_addr, NULL);
			return (1);
    	}
    }
    else
    {
		/* Dann Nachricht aus der Datei lesen */
		mbuf.next=mbuf.start;
    	if (rfc_read_message(inputfilename,&mbuf))
    	{
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, strerror (errno));
			errormail (NULL,
			   "weird: cannot read input file",
			   strerror (errno),
			   gm_addr, NULL);
			return 1;
    	}
    }
	msgid=rfc_find_header(mbuf.start,"message-id");
	if (!msgid)
	{
		errormail (NULL,
		   "weird: news without Message-Id?",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
	}
    /* 
     * Die Nachricht ist also vorhanden 
     * Jetzt Absender und Gruppe feststellen.
     */
    from=rfc_find_header(mbuf.start,"from");
    if (!from)
    {
		errormail (NULL,
		   "weird: news without From?",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
    }
    ml=NULL;
    if (rfc_adress_to_metalist(1,&ml,from)||!ml || !ml->text)
    {
		errormail (NULL,
		   "weird: cannot parse From header",
		   strerror (errno),
		   gm_addr, NULL);
		return 1;
    }
    strcpy(from,ml->text);

	if (is_routing)
	{
		route=route_it(from,NULL);
		if (argument_test)
		{
			if (route)
				printf(_("route for news from %s via %s\n"),from, route);
			else
				printf(_("no route for news from %s\n"),from);
			return 0;
		}
	    if (!route)
	    {
	    	/* das bedeutet, da wir noch nicht einmal wissen, ob
	    	 * die Nachricht ber ein Gate oder lokal kam. Die 
	    	 * Nachricht darf auch nicht "auf Verdacht" gebounced
	    	 * werden (wg. Gatewaybetrieb).
	    	 * Der Fall, da keine Route gefunden wird, kommt sehr hufig
	    	 * vor, wenn man Nachrichten aus dem Usenet bekommt mit
	    	 * Crosspostings in Maus-Gruppen. Von daher kann man die
	    	 * Warnung ausschalten :-)
	    	 */
	    	if (opts.warn_news_routing)
	    	{
				errormail (NULL,
					"weird: cannot find a route for news from",
					from,
				   gm_addr, NULL);
			}
			msglog("# %s: no route, from %s",msgid,from);
	    	return 1;
	    }
	    action=route;
	} else if (!is_allowed(from)) {
		if (opts.warn_news_routing) {
			errormail (NULL,
					   "weird: cannot find a route for news from",
					   from,
					   gm_addr, NULL);
		}
		msglog("# %s: no route, from %s",msgid,from);
		return 1;
	}

	/* Diverse Formen von Control-Headern bearbeiten */
	{	
		char *control;
		char *also_control=NULL;
		char *supersedes=NULL;
		char *vgl;
		control=rfc_find_header(mbuf.start,"control");
		if (!control)
			also_control=rfc_find_header(mbuf.start,"also-control");
		if (!control && !also_control)
			supersedes=rfc_find_header(mbuf.start,"supersedes");
		if (control || also_control || supersedes)
		{
			int exit_after=0;
			if (control)
				exit_after=1;
			if (supersedes)	
			{
				work_cancel(supersedes,from);
				msglog("  %s: supersedes added",msgid);
			}
			if (control)
			{
				vgl=control;
				if (!strncasecmp(vgl,"cancel ",7))
				{
					msglog("  %s: control cancel added",msgid);
					work_cancel(vgl+7,from);
				}
				exit_after=1;
			}
			if (also_control)
			{
				vgl=also_control;
				if (!strncasecmp(vgl,"cancel ",7))
				{
					msglog("  %s: also-control cancel added",msgid);
					work_cancel(vgl+7,from);
				}
			}
			if (exit_after)
				return 0;
		}
	}

	ok=0;
	fd=-1; /* keep gcc happy */
	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/%ld", 
			opts.spoolbase, action, (long) id);
		if (!inputfilename)
		{
			fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
			if (fd>=0)
				ok=1;
		}
		else
		{
#ifdef HAVE_SYMLINK
			if (symlink(inputfilename,fname)==0)
				ok=1;
			else
#endif
				if (file_copy(inputfilename,fname))
					ok=1;
		}
	} while (i<10 && !ok);

	if (!ok)
	{
		if (inputfilename)
		{
			errormail (NULL,
				"weird: cannot spool news",
				strerror(errno),
			   gm_addr, NULL);
		}
		else
		{
			errormail (NULL,
				"weird: cannot create symlink",
				strerror(errno),
			   gm_addr, NULL);
		}
		msglog("# %s: cannot spool",msgid);
		return(1);
	}
	if (inputfilename)
	{
		msglog("# %s: news spooled as %ld",msgid,id);
		return 0;
	}
	if (xwrite(fd,mbuf.start,strlen(mbuf.start))<0)
	{
		close(fd);
		unlink(fname);
		/* News smart bouncen */
		errormail (NULL,
		   "weird: cannot spool message",
		   strerror (errno),
		   gm_addr, NULL);
		msglog("# %s: cannot spool",msgid);
		return 1;
	}
	close(fd);
	msglog("# %s: news spooled as %ld",msgid,id);
	return 0;
}

static int 
handle_rnews(const char *inputfilename)
{
	char fname[PATH_MAX];
	int fdin;
	int fdout;
	int i;
	unsigned long id;
	char buf[64*1024];
	size_t got;

	/* einen Newsbatch nehmen und komplett spoolen */

	i=0;
	do
	{
		/* 
		 * eine Transfer-ID holen 
		 */
		id = get_transfer_id ();
		/* 
		 * In welches File spoolen? 
		 */
		sprintf (fname, "%s/%s/outgoing/nb%lu", 
			opts.spoolbase, action, id);
		fdout=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
		i++;
	} while (i<10 && fdout<0);

	if (fdout<0)
	{
		const char *es=strerror(errno);
		errormail (NULL,
			"weird: cannot spool newsbatch",
			es,
		   gm_addr, NULL);
		msglog("# [%ld]: cannot open spoolfile: %s",id,es);
		return(1);
	}

	if (inputfilename == NULL)
		fdin=STDIN_FILENO;
	else
	{
		fdin=open(inputfilename,O_RDONLY);
		if (fdin<0)
		{
			const char *es=strerror(errno);
			/* Hm? Eingabedatei nicht da? */
			log(LOG_ERR,"%s: %s\n", inputfilename, es);
			msglog("# [%ld]: cannot open inputfile %s: %s",id,
				   inputfilename, es);
			errormail (NULL,
			   "weird: cannot read input file",
			   es,
			   gm_addr, NULL);
			return 1;
    	}
    }

    while ((got=read(fdin,buf,sizeof(buf)))>0)
    {
		if (xwrite(fdout,buf,got)<0)
		{
			const char *es=strerror(errno);
			close(fdout);
			unlink(fname);

			errormail (NULL,
			   "weird: cannot spool newsbatch",
			   es,
			   gm_addr, NULL);
			msglog("# [%ld]: cannot write to spoolfile: %s",id, es);
			return 1;
		}
    }
	msglog("# [%ld]: rnews spooled",id);
	close(fdin);
	return 0;
}

static void
test_routing(void)
{
	char from[1024];
	char to[1024];
	char *p;
	char *route;
	while (1)
	{
		printf(_("from> "));
		if (!fgets(from,sizeof(from),stdin))
			break;
		p=strpbrk(from,"\r\n");
		if (p)
			*p='\0';
		if (!*from)
			break;
		printf(_("  to> "));
		if (!fgets(to,sizeof(to),stdin))
			break;
		p=strpbrk(to,"\r\n");
		if (p)
			*p='\0';
		if (!*to)
			break;
		route=route_it(strtrim(from),strtrim(to));
		if (route)
			printf(_(" via %s\n"),route);
		else
			printf(_(" no way\n"));
	}
}

#define WHAT_MAIL 1
#define WHAT_RMAIL 2
#define WHAT_RSMTP 3
#define WHAT_NEWS 4
#define WHAT_RNEWS 5
#define WHAT_ROUTE_SHOW 6
#define WHAT_ROUTE_TEST 8

static void
usage(int exitcode, const char *what)
{
    FILE *f=stdout;

    if (exitcode)
    {
        if (what)
            fprintf(stderr, "%s: %s\n",program_name,what);
        fprintf (stderr, _("Try `%s --help' for more information.\n"),
            program_name);
        exit(exitcode);
    }

    fprintf(f, "%s - %s\n", program_name, PACKAGE_VERSION);

    fprintf(f,_("Usage: %s [options] file ...\n"), program_name);
    fputs(_("Spool messages for transport to MausTausch-BBS\n"),f);
	fputs(_(
"  -C  --channel CHANNEL       use channel CHANNEL\n"
"  -c  --cnf-file FILE         use config file FILE\n"
"  -f  --from FROM             mail is from FROM\n"
"      --help                  this text\n"
"  -i, --input FILE            read input (outfile) from FILE\n"
"  -m, --mail                  take just one mail\n"
"      --news                  take just one news\n"
"      --rmail                 work as rmail\n"
"      --rnews                 work as rnews\n"
"      --route-show            show routing database\n"
"      --route-test            interactively test routing database\n"
"      --rsmtp                 work as rsmtp\n"
"  -T  --to TO                 mail to TO\n"
"      --test                  test, dont spool\n"
"      --version               show version info\n"
"\n"
"short options use the same arguments as the long ones\n"
    ),f);
    exit(exitcode);
}

static void
usage1(int ecode)
{
    usage(ecode,NULL);
}

int
main (int argc, char **argv)
{
	int ret;
   	int what_i_do=0;
   	BOOLEAN override_action=FALSE;

	const char *cnffname=SYSCONFDIR "/" CNFNAME;
	const char *argument_to=NULL;
	const char *argument_from=NULL;
	const char *inputfilename=NULL;

	struct option cmdlinopts[] =
	{
		{"cnffile", required_argument, NULL, 'c'},
		{"cnf-file", required_argument, NULL, 'c'},
		{"action",required_argument,NULL,'a'},
		{"channel",required_argument,NULL,'C'},
		{"from", required_argument, NULL, 'f'},
		{"input", required_argument, NULL, 'i'},
		{"mail",no_argument,&what_i_do,WHAT_MAIL},
		{"rmail",no_argument,&what_i_do,WHAT_RMAIL},
		{"rnews",no_argument,&what_i_do,WHAT_RNEWS},
		{"news",no_argument,&what_i_do,WHAT_NEWS},
		{"route-show", 0, &what_i_do, WHAT_ROUTE_SHOW},
		{"route-test", 0, &what_i_do, WHAT_ROUTE_TEST},
		{"rsmtp",no_argument,&what_i_do,WHAT_RSMTP},
		{"test",no_argument,&argument_test,1},
		{"to", required_argument, NULL, 'T'},

		{NULL, 0, NULL, 0}
	};
	int option_index=0;
	const char *slash;

	program_name=argv[0];
	slash=strrchr(program_name,'/');
	if (slash)
		program_name=slash+1;

	parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage1);

	while (1)
	{
		int c;

		c = getopt_long (argc, argv, "a:C:c:f:i:mT:",
			cmdlinopts, &option_index);
		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch (c)
		{
		case 'C':
		case 'a': action=optarg; break;
		case 'c': cnffname = optarg; break;
		case 'i': inputfilename = optarg; break;
		case 'f': argument_from = optarg; break;
		case 'm': what_i_do=WHAT_MAIL; break;
		case 'T': argument_to = optarg; break;
		case '?':
			log (LOG_ERR, _("unknown option %s"), argv[optind - 1]);
			usage (2, _("unknown option"));
			break;
		default:
			break;
		}
	}

	if (what_i_do==0)
	{
		if (!strcmp(program_name,"rnews") || !strcmp(program_name,"mrnews"))
			what_i_do=WHAT_RNEWS;
		if (!strcmp(program_name,"mmail"))
			what_i_do=WHAT_MAIL;
		if (!strcmp(program_name,"rmail") || !strcmp(program_name,"mrmail"))
			what_i_do=WHAT_RMAIL;
		if (!strcmp(program_name,"rsmtp") || !strcmp(program_name,"mrsmtp"))
			what_i_do=WHAT_RSMTP;
		if (!strcmp(program_name,"mnews"))
			what_i_do=WHAT_NEWS;

		if (!what_i_do)
		{
			log(LOG_ERR, _("%s: don't know what to do"),program_name);
			exit(1);
		}
	}

	switch(what_i_do)
	{
	case WHAT_RMAIL:
	 	/* rmail to1 [to2 ...] */
		if (!argv[optind])
		{
			log (LOG_ERR, _("no address given"));
			exit(2);
		}
		if (!action)
		{
			action="rmail";
   			override_action=TRUE;
   		}
		break;
	case WHAT_RSMTP:
	 	/* rsmtp [action] */
	 	if (argv[optind])
	 	{
			if (argv[optind+1])
			{
				log (LOG_ERR, _("too many arguments"));
				exit (2);
			}
    		action=argv[optind];
	 	}
    	else if (!action)
    	{
    		action="rsmtp"; /* Default */
   			override_action=TRUE;
   		}
		break;
	case WHAT_RNEWS:
		/* rnews [inputfile] */
		if (!action)
		{
			action="rnews";
			override_action=TRUE;
		}
		if (argv[optind])
		{
			if (argv[optind+1])
			{
				log (LOG_ERR, _("too many arguments"));
				exit (2);
			}
    		if (inputfilename)
				log (LOG_WARNING, 
					 _("warning: given filename overrides --input"));
			inputfilename=argv[optind];
			if (!strcmp(inputfilename,"-"))
				inputfilename=NULL;
    	}
    	break;
	case WHAT_MAIL:
	case WHAT_NEWS:
		/* mail {action | "route"} */
	 	/* news {action | "route"} */
		if (!argv[optind])
		{
			if (!action) {
				log (LOG_ERR, _("channel not given"));
				exit(2);
			}
		}
		else if (argv[optind+1])
		{
			log (LOG_ERR, _("too many arguments"));
			exit (2);
		}
		else
			action=argv[optind];
		break;
	case WHAT_ROUTE_SHOW:
		if (!argv[optind])
		{
			log (LOG_ERR, _("channel not given"));
			exit(2);
		}
		else if (argv[optind+1])
		{
			log (LOG_ERR, _("too many arguments"));
			exit (2);
		}
		action=argv[optind];
		break;
	case WHAT_ROUTE_TEST:
		action="route";
		break;
	}

	if (!strcasecmp(action,"route"))
		is_routing=1;

    do_conf(cnffname,action ? action : "route" );

	if (override_action)
	{
		if (what_i_do== WHAT_RMAIL)
			action=opts.rmail_route;
		else if (what_i_do==WHAT_RSMTP)
			action=opts.rsmtp_route;
		else if (what_i_do==WHAT_RNEWS)
			action=opts.rnews_route;
		if (!action)
		{
			log (LOG_ERR, _("indirect channel not set"));
			exit(2);
		}
	}

	error_to[0]='\0';
	sprintf (gm_addr, "=%s", opts.gatemaster);

	openlog (program_name, 0, opts.sl_facility);
	setlogmask(LOG_UPTO(opts.sl_priority));
	open_msglog();

	if (route_open(opts.spoolbase))
	{
		log(LOG_ERR,_("cannot open routing database"));
		exit(1);
	}

	if (what_i_do==WHAT_ROUTE_SHOW)
	{
		ret=route_show(action);
	}
	else if (what_i_do==WHAT_ROUTE_TEST)
	{
		test_routing();
		ret=0;
	}
	else if (what_i_do==WHAT_RMAIL)
	{
		ret=handle_rmail(inputfilename,&argv[optind]);
	}
	else if (what_i_do==WHAT_MAIL)
	{
		ret=handle_mail(inputfilename,argument_from,argument_to);
	}
	else if (what_i_do==WHAT_RSMTP)
	{
		ret=handle_rsmtp(inputfilename);
	}
	else if (what_i_do==WHAT_RNEWS)
		ret=handle_rnews(inputfilename);	
	else /* WHAT_NEWS */
		ret=handle_news(inputfilename);	
	if (got_cancels)
		cancel_end(0,1);

	closelog ();
	close_msglog();

	exit (ret);
}

static void 
cleanup_empfaenger(struct empfaenger **anker)
{
	struct empfaenger *p,*next;
	p=*anker;
	while (p)
	{
		next=p->next;
		if (p->adr)
			free(p->adr);
		free(p);
		p=next;
	}
	*anker=NULL;
}

static int 
handle_rsmtp(const char *inputfilename)
{
	FILE *f=stdin;
	char *p;
	char buf[1024];
	size_t l;
	int retcode=0;
	BOOLEAN got_greeting=FALSE;
	BOOLEAN got_ehlo=FALSE;
	BOOLEAN got_data=FALSE;
	unsigned long errors=0;
	char from[1024]="";
	BOOLEAN got_from=FALSE;
	struct empfaenger *anker=NULL;
	buf_t body={NULL,NULL,0};
	BOOLEAN in_header=FALSE;

	if (inputfilename)
	{
		f=fopen(inputfilename,"r");
		if (!f)
		{
			log(LOG_ERR,_("%s: cannot open %s: %s"), "mrsmtp",
				inputfilename,strerror(errno));
			return -1;
		}
	}

	while (1)
	{
		if (!fgets(buf,sizeof(buf),f))
		{
			retcode=-1;
			log(LOG_ERR,_("mrsmtp: unexpected end of transmission: %s"),	
				strerror(errno));
			break;
		}
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
puts(buf);
		if (!got_greeting)
		{
			if (!strncasecmp(buf,"HELO ",5))
			{
				got_greeting=TRUE;
			}
			else if (!strncasecmp(buf,"EHLO ",5))
			{
				got_greeting=TRUE;
				got_ehlo=TRUE;
			}
			else 
				errors++; 
			continue;
		}
		if (!got_data)
		{
			if (!strncasecmp(buf,"MAIL FROM:",10))
			{
				char *adr=buf+10;
				while (isspace(*adr))
					adr++;
				if (*adr=='<')
				{
					adr++;
					while (isspace(*adr))
						adr++;
				}
				p=strpbrk(adr,"\t ");
				if (p)
					*p=0;
				strcpy(from,adr);
				p=strchr(from,'>');
				if (p)
					*p=0;
				cleanup_empfaenger(&anker);
				got_from=TRUE;
				continue;
			}
			if (!strncasecmp(buf,"RCPT TO:",8))
			{
				char *adr=buf+8;
				struct empfaenger *new;

				if (!got_from)
				{
					/* nicht nur ein kleines Problem: Es gibt noch nicht 
					 * mal eine Adresse, an die die Fehlermeldung geschickt
					 * werden kann.
					 */
					log(LOG_NOTICE,_("got smtp without MAIL FROM"));
					/* 
					 * Bser Hack, mglicherweise ein Fehler?
					 */
					strcpy(from,"unknown");
					continue;
				}
				while (isspace(*adr))
					adr++;
				if (*adr=='<')
				{
					adr++; /* wegen '<' */
					while (isspace(*adr))
						adr++;
				}
				p=strpbrk(adr,"\t ");
				if (p)
					*p=0;
				p=strchr(adr,'>');
				if (p)
					*p=0;
				new=XMALLOC(sizeof(struct empfaenger));
				new->adr=XSTRDUP(adr);
				new->next=anker;
				anker=new;
				continue;
			}
			if (!strcasecmp(buf,"DATA"))
			{
				if (!anker)
				{
					log(LOG_NOTICE, _("no receiver given -> skipped"));
					continue;
				}
				if (!got_from)
				{
					log(LOG_NOTICE,_("no sender given -> skipped"));
					continue;
				}
				in_header=TRUE;
				got_data=1;
				continue;
			}
			if (!strcasecmp(buf,"QUIT"))
			{
				cleanup_empfaenger(&anker);
				break;
			}
			continue;
		}
		/* got_data gesetzt */
		if (*buf=='.')
		{
			if (!buf[1])
			{
				finish_rsmtp(from,anker,&body);
				got_data=FALSE;
				body.next=body.start;
				*body.start=0;
				continue;
			}
			p=buf+1;
		}
		else
			p=buf;
		l=strlen(p);
		if (!body.start || body.next-body.start+1+l >body.size)
		{
			size_t plus=64*1024;
			size_t off=body.next-body.start;
			if (l+1>plus)
				plus=l+1;
			body.start=XREALLOC(body.start,plus+body.size);
			body.size+=plus;
			body.next=body.start+off;
		}
		if (!*p)
			in_header=FALSE;
		if (in_header && isspace(*p) && body.next-body.start>2)
		{
			/* Fortsetzungszeile */
			strcpy(body.next-2,p);
			body.next+=l-2;
		}
		else
		{
			strcpy(body.next,p);
			body.next+=l;
		}
		strcpy(body.next,"\r\n");
		body.next+=2;
	}
	free(body.start);
	return retcode;
}


static void 
finish_rsmtp(char *from, struct empfaenger *emp, buf_t *message)
{
	char env_buf[2048];
	char fname[PATH_MAX];
	long id;
	const char *msgid;
	struct empfaenger *err_route_list=NULL;
	struct empfaenger *err_spool_list=NULL;
	char *route;

	msgid=rfc_find_header(message->start,"message-id");
   	if (!msgid)
   		msgid="[mroute: unknown id]";

	while (emp)
	{
		route=route_it(from,emp->adr);
		if (!route && opts.reroute_host)
		{
			/* Mail via eine andere Kiste schicken */
			char adr[1024];
			char cmd[2048];
			char *p;
			FILE *wfd;
			strcpy(adr,emp->adr);
			p=strchr(adr,'@');
			if (p)
				*p='%';
			strcat(adr,"@");
			strcat(adr,opts.reroute_host);
			sprintf(cmd,"rmail '%s'",adr);
			wfd=popen(cmd,"w");
			if (wfd)
			{
				fwrite(message->start,strlen(message->start),1,wfd);
				msglog("# %s: routing failed, rerouted via %s",
					msgid,opts.reroute_host);
				pclose(wfd);
			}
			else
				msglog("# %s: routing failed, rerouting also with %s",
					msgid,strerror(errno));
		}
		else if (!route)
		{
			struct empfaenger *n=XMALLOC(sizeof(struct empfaenger));
			n->adr=emp->adr;
			n->next=err_route_list;
			err_route_list=n;
			msglog("# %s: routing failed", msgid);
		}
		else 
		{
			int i=0;
			int fd;
			action=route;
			do
			{
				/* 
				 * eine Transfer-ID holen 
				 */
				id = get_transfer_id ();
				/* 
				 * In welches File spoolen? 
				 */
				sprintf (fname, "%s/%s/outgoing/mail%ld", 
					opts.spoolbase, action, (long) id);
				fd=open(fname,O_WRONLY|O_CREAT|O_EXCL,0640);
			} while (i<100 && fd<0);

			if (fd<0)
			{
				if (from && *from)
				{
					struct empfaenger *n=XMALLOC(sizeof(struct empfaenger));
					n->adr=emp->adr;
					n->next=err_spool_list;
					err_spool_list=n;
					errormail (NULL,
					   "weird: cannot open spool file",
					   strerror (errno),
					   gm_addr, NULL);
				}
				msglog("# %s: cannot spool as %ld",msgid,id);
			}
			else
			{

				sprintf(env_buf,
					"X-MausConnect-Env-To: %s\n"
					"X-MausConnect-Env-From: %s\n",
					emp->adr,
					from);

				if (xwrite(fd,env_buf,strlen(env_buf))<0 
					|| xwrite(fd,message->start,strlen(message->start))<0)
				{
					struct empfaenger *n=XMALLOC(sizeof(struct empfaenger));
					n->adr=emp->adr;
					n->next=err_spool_list;
					err_spool_list=n;
					unlink(fname);
					errormail (NULL,
					   "weird: cannot spool mail",
					   strerror (errno),
					   gm_addr, NULL);
					msglog("# %s: cannot spool as %ld (write error)",msgid,id);
				}
				close(fd);
				msglog("  %s: rsmtp spooled as %ld",msgid,id);
			}

		}
		emp=emp->next;
	}

	if (err_route_list || err_spool_list)
	{
		struct empfaenger *next;
		char buf[2048];
		size_t bufl=0;
		size_t l;

		if (err_route_list)
		{
			bufl+=sprintf(buf,"You are not allowed to use the gateway to:");
			while (err_route_list)
			{
				l=strlen(err_route_list->adr);
				if (l+3+bufl<sizeof(buf))
					bufl+=sprintf(buf+bufl,"%s\n",err_route_list->adr);
				next=err_route_list;
				free(err_route_list->adr);
				free(err_route_list);
				err_route_list=next;
			}
		}
		if (err_spool_list)
		{
			bufl+=sprintf(buf,"Couldn't spool message to:");
			while (err_spool_list)
			{
				l=strlen(err_spool_list->adr);
				if (l+3+bufl<sizeof(buf))
					bufl+=sprintf(buf+bufl,"%s\n",err_spool_list->adr);
				next=err_spool_list;
				free(err_spool_list->adr);
				free(err_spool_list);
				err_spool_list=next;
			}
		}
		errormail (NULL,
		   "rsmtp prozessing error(s)",
		   buf,
		   gm_addr,from, NULL);
	}
}

